@@ -4,15 +4,12 @@ from __future__ import annotations
 
 import hashlib
 import os
+import stat
 import time
+from importlib.util import find_spec
 
 
-try:
-    from platformdirs import user_data_dir
-except ImportError:  # pragma: no cover
-    CACHE_AVAILABLE = False
-else:
-    CACHE_AVAILABLE = True
+CACHE_AVAILABLE = find_spec('platformdirs') is not None
 
 
 class Cache:
@@ -37,13 +34,19 @@ class Cache:
         return hashlib.blake2b(url.encode(), digest_size=16).digest().hex()
 
     def read_file(self, fpath: str, encoding: str = 'utf-8') -> str:  # noqa: D102
-        with open(fpath, encoding=encoding) as f:
-            return f.read().split('\n', 1)[1]
+        f = open(fpath, encoding=encoding)  # noqa: SIM115
+        content = f.read().split('\n', 1)[1]
+        f.close()
+        return content
 
     def get_(self, url: str, encoding: str = 'utf-8') -> str | None:  # noqa: D102
         key = self.generate_unique_key_from_url(url)
         fpath = os.path.join(self.cache_dir, key)
-        if os.path.isfile(fpath):
+        try:
+            is_file = stat.S_ISREG(os.stat(fpath).st_mode)
+        except (FileNotFoundError, OSError):  # pragma: no cover
+            return None
+        if is_file:
             creation_time = self.get_creation_time_from_fpath(fpath)
             if time.time() < creation_time + self.expiration_seconds:
                 return self.read_file(fpath, encoding=encoding)
@@ -72,9 +75,13 @@ def get_cache_directory() -> str | None:
     if not CACHE_AVAILABLE:
         return None
 
+    try:
+        from platformdirs import user_data_dir
+    except ImportError:
+        return None
+
     cache_dir = user_data_dir('mkdocs-include-markdown-plugin')
-    if not os.path.isdir(cache_dir):
-        os.makedirs(cache_dir)
+    os.makedirs(cache_dir, exist_ok=True)
 
     return cache_dir
 
